/* main.c - (C) 1998 Tomas Berndtsson (tomas@nocrew.org)
 *
 *  This piece of software is released under the terms of the GNU General
 *  Public Licence version 2 or, at your choice, any higher version.
 *  The GNU GPL is included with this package in the file COPYING.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include "version.h"

#include "main.h"
#include "fb.h"

#ifdef HAVE_LIBMAGICK
#include "magick.h"
#endif /* HAVE_LIBMAGICK */
#ifdef HAVE_LIBJPEG
#include "jpeg.h"
#endif /* HAVE_LIBJPEG */

int parse_options(int, char **, j_settings *);

int main(int argc, char *argv[])
{
  int arg;
  FILE *jpegfile;
  j_settings js;
  fb_picture *pic=NULL;
  int mode;

#ifndef HAVE_IMAGE_LIB
  fprintf(stderr, "Sorry, no image processing library was "
	  "compiled into this version.\n");
  exit(1);
#endif /* HAVE_IMAGE_LIB */

  mode = USING_LIBMAGICK;

  arg = parse_options(argc, argv, &js);

  if(arg >= argc) {
    fprintf(stderr,"No input file!\n");
    exit(1);
  }

  while(arg < argc) {
    if(!js.force_magick) {
      if(!strcmp(argv[arg], "-")) {
	if(js.verbose)
	  fprintf(stderr, "Reading from stdin...\n");
	jpegfile = stdin;
      } else {
	if(js.verbose)
	  fprintf(stderr, "Reading %s...\n", argv[arg]);
	if((jpegfile=fopen(argv[arg], "rb")) == NULL) {
	  perror(argv[arg]);
	  arg++;
	  continue;
	}
      }
#ifdef HAVE_LIBJPEG
      mode = USING_LIBJPEG;
      if((pic=j_decompress(jpegfile, &js)) == NULL) {
#ifdef HAVE_LIBMAGICK
	if(js.verbose)
	  fprintf(stderr,"%s: Not a jpeg, trying with libmagick\n", argv[0]);
	mode = USING_LIBMAGICK;
#endif /* HAVE_LIBMAGICK */
      }
      
      if(jpegfile != stdin)
	fclose(jpegfile);
#endif /* HAVE_LIBJPEG */
    }
#ifdef HAVE_LIBMAGICK
    if(mode == USING_LIBMAGICK) {
      if((pic=im_convert(argv[arg], &js)) == NULL) {
	fprintf(stderr,"%s: Couldn't convert image %s\n", argv[0], argv[arg]);
	arg++;
	continue;
      }
    }
#endif /* HAVE_LIBMAGICK */

    if(pic == NULL) {
      fprintf(stderr, "%s: Couldn't convert image %s\n", argv[0], argv[arg]);
      arg++;
      continue;
    }

    if(js.wait_key) {
      fprintf(stderr,"Press Return to view image.");
      getchar();
    }

    if(fb_viewpic(pic, mode, &js))
      break;
	
    arg++;
  }
  
  if(!js.switchfb)
    fb_exit();

  return 0;
}

extern char *optarg;
extern int optind, opterr, optopt;

int parse_options(int argc, char *argv[], j_settings *js)
{
  int arg;

  struct option long_options[] = { 
    { "two-pass", no_argument, NULL, '2' },
#ifdef HAVE_LIBMAGICK
    { "magick", no_argument, NULL, 'm' },
#endif /* HAVE_LIBMAGICK */
    { "brightness", required_argument, NULL, 'b' },
    { "tree-depth", required_argument, NULL, 't' },
    { "verbose", no_argument, NULL, 'v' },
    { "wait-key", no_argument, NULL, 'k' },
    { "scale-down", required_argument, NULL, 'd' },
    { "scale-up", required_argument, NULL, 'u' },
    { "dont-switch", no_argument, NULL, 's' },
    { "idct-method", required_argument, NULL, 'i' },
    { "help", no_argument, NULL, 'H' },
    { "version", no_argument, NULL, 'V' } };

#ifdef HAVE_LIBMAGICK
  char *short_options = "2kb:t:vd:u:mi:s";
#else /* HAVE_LIBMAGICK */
  char *short_options = "2kb:t:vd:u:i:s";
#endif /* HAVE_LIBMAGICK */

  /* default settings */
  js->wait_key = js->force_magick = js->verbose = js->two_pass = 0;
  js->switchfb = 1;
  js->scale_num = js->scale_denom = 1;
  js->idct_method = 0; /* JDCT_ISLOW */
  js->bitplanes = 8;
  js->tree_depth = 8;
  js->brightness = 1.0;

  while((arg=getopt_long(argc, argv, short_options, long_options, NULL)) != EOF) {
    switch(arg) {
    case '2':
      js->two_pass = 1;
      break;
#ifdef HAVE_LIBMAGICK
    case 'm':
      js->force_magick = 1;
      break;
#endif /* HAVE_LIBMAGICK */
    case 'b':
      js->brightness = atof(optarg);
      break; 
    case 't':
      js->tree_depth = atoi(optarg);
      break;
    case 'd':
      js->scale_denom = atoi(optarg);
      break;
    case 'u':
      js->scale_num = atoi(optarg);
      break;
    case 'i':
      js->idct_method = atoi(optarg);
      break;
    case 'v':
      js->verbose = 1;
      break;
    case 'k':
      js->wait_key = 1;
      break;
    case 's':
      js->switchfb = 0;
      break;
    case 'H':
      fprintf(stderr,"Usage: %s [OPTION(S)] <filename>\n"
	      "\n"
	      "valid options are:\n"
	      "-2, --two-pass        perform two-pass quantize (slower)\n"
	      "-d, --scale-down n    scale down image with factor n (even 2 powers)\n"
	      "-u, --scale-up n      scale up image with factor n (doesn't work)\n"
	      "-b, --brightness f    set brightness percentage. (0.0 - black, 1.0 - normal)\n"
	      "-t, --tree-depth n    tree depth for quantize, default 8\n"
	      "-s, --dont-switch     Don't switch back to console between images\n"
	      "-i, --idct-method n   IDCT method to use with libjpeg.\n"
	      "                      (0 = Slow integer, 1 = Fast integer, 2 = Float)\n"
	      "-v, --verbose         print informational messages\n"
#ifdef HAVE_LIBMAGICK
	      "-m, --magick          force use of libmagick instead of libjpeg\n"
#endif /* HAVE_LIBMAGICK */
	      "-k, --wait-key        wait for Return before viewing\n"
	      "    --version         print version and exit\n"
	      "    --help            print this text and exit\n"
	      "\n"
	      "If <filename> is `-', stdin is used as input image.\n"
	      "\n"
	      "Keycommands during viewing: (u)p, (d)own, (l)eft, (r)ight moves half screens\n"
	      "                            (U)p, (D)own, (L)eft, (R)ight moves full screens\n"
	      "                            'p' sets the palette again\n"
	      "                            Space or return exits\n"
              ,argv[0]);
      exit(1);
      break;
    case 'V':
      fprintf(stderr,PROGRAM_NAME " version %d.%d.%d%s\n", 
	      PROGRAM_VERSION_MAJOR, PROGRAM_VERSION_MINOR,
	      PROGRAM_VERSION_SUBMINOR, PROGRAM_VERSION_EXTENSION);
      fprintf(stderr, 
	      "Compiled with: "
#ifdef HAVE_LIBJPEG
	      "libjpeg "
#endif /* HAVE_LIBJPEG */
#ifdef HAVE_LIBMAGICK
	      "libMagick "
#endif /* HAVE_LIBMAGICK */
	      "\n");
      exit(1);
      break;
    default:
    }
  }

  return optind;
}



